# -*- coding: utf-8 -*-
#
# Copyright © 2013 Spyder Project Contributors
# Licensed under the terms of the MIT License
# (see LICENSE.txt for details)
"""Spyder unitest plugin."""

# Local imports
from .unittestplugin import UnitTestPlugin as PLUGIN_CLASS

__version__ = '0.3.0'
PLUGIN_CLASS



# -*- coding: utf-8 -*-
#
# Copyright © 2017 Spyder Project Contributors
# Licensed under the terms of the MIT License
# (see LICENSE.txt for details)
"""Tests for jsonstream.py"""

# Standard library imports
from io import StringIO, TextIOBase

# Local imports
from spyder_unittest.backend.jsonstream import (JSONStreamReader,
                                                JSONStreamWriter)

try:
    from unittest.mock import create_autospec
except ImportError:
    from mock import create_autospec  # Python 2


def test_jsonstreamwriter_with_list():
    stream = StringIO()
    writer = JSONStreamWriter(stream)
    writer.write([1, 2])
    assert stream.getvalue() == '6\n[1, 2]\n'


def test_jsonstreamwriter_with_unicode():
    stream = StringIO()
    writer = JSONStreamWriter(stream)
    writer.write(u'三')  # u prefix for Python2 compatibility
    assert stream.getvalue() == '8\n"\\u4e09"\n'


def test_jsonstreamwriter_flushes():
    stream = create_autospec(TextIOBase)
    writer = JSONStreamWriter(stream)
    writer.write(1)
    stream.flush.assert_called_once_with()


def test_jsonstreamreader_with_list():
    reader = JSONStreamReader()
    assert reader.consume('6\n[1, 2]\n') == [[1, 2]]


def test_jsonstreamreader_with_windows_lineending():
    reader = JSONStreamReader()
    assert reader.consume('6\r\n[1, 2]\r\n') == [[1, 2]]


def test_jsonstreamreader_with_unicode():
    reader = JSONStreamReader()
    assert reader.consume('8\n"\\u4e09"\n') == [u'三']


def test_jsonstreamreader_with_partial_frames():
    reader = JSONStreamReader()
    txt = '1\n2\n' * 3
    assert reader.consume(txt[:2]) == []
    assert reader.consume(txt[2:-2]) == [2, 2]
    assert reader.consume(txt[-2:]) == [2]


def test_jsonsteamreader_writer_integration():
    stream = StringIO()
    writer = JSONStreamWriter(stream)
    reader = JSONStreamReader()
    writer.write([1, 2])
    writer.write({'a': 'b'})
    assert reader.consume(stream.getvalue()) == [[1, 2], {'a': 'b'}]



# -*- coding: utf-8 -*-
#
# Copyright © 2017 Spyder Project Contributors
# Licensed under the terms of the MIT License
# (see LICENSE.txt for details)
"""Keep track of testing frameworks and create test runners when requested."""


class FrameworkRegistry():
    """
    Registry of testing frameworks and their associated runners.

    The test runner for a framework is responsible for running the tests and
    parsing the results. It should implement the interface of RunnerBase.

    Frameworks should first be registered using `.register()`. This registry
    can then create the assoicated test runner when `.create_runner()` is
    called.

    Attributes
    ----------
    frameworks : dict of (str, type)
        Dictionary mapping names of testing frameworks to the types of the
        associated runners.
    """

    def __init__(self):
        """Initialize self."""
        self.frameworks = {}

    def register(self, runner_class):
        """Register runner class for a testing framework.

        Parameters
        ----------
        runner_class : type
            Class used for creating tests runners for the framework.
        """
        self.frameworks[runner_class.name] = runner_class

    def create_runner(self, framework, widget, tempfilename):
        """Create test runner associated to some testing framework.

        This creates an instance of the runner class whose `name` attribute
        equals `framework`.

        Parameters
        ----------
        framework : str
            Name of testing framework.
        widget : UnitTestWidget
            Unit test widget which constructs the test runner.
        resultfilename : str or None
            Name of file in which to store test results. If None, use default.

        Returns
        -------
        RunnerBase
            Newly created test runner

        Exceptions
        ----------
        KeyError
            Provided testing framework has not been registered.
        """
        cls = self.frameworks[framework]
        return cls(widget, tempfilename)



# -*- coding: utf-8 -*-
#
# Copyright © 2017 Spyder Project Contributors
# Licensed under the terms of the MIT License
# (see LICENSE.txt for details)
"""Tests for unittestrunner.py"""

# Local imports
from spyder_unittest.backend.runnerbase import Category
from spyder_unittest.backend.unittestrunner import UnittestRunner


def test_unittestrunner_load_data():
    output = """test_isupper (teststringmethods.TestStringMethods) ... ok
test_split (teststringmethods.TestStringMethods) ... ok
extra text\n"""
    runner = UnittestRunner(None)
    res = runner.load_data(output)
    assert len(res) == 2

    assert res[0].category == Category.OK
    assert res[0].status == 'ok'
    assert res[0].name == 'teststringmethods.TestStringMethods.test_isupper'
    assert res[0].message == ''
    assert res[0].extra_text == []

    assert res[1].category == Category.OK
    assert res[1].status == 'ok'
    assert res[1].name == 'teststringmethods.TestStringMethods.test_split'
    assert res[1].message == ''
    assert res[1].extra_text == ['extra text\n']


def test_unittestrunner_load_data_removes_footer():
    output = """test1 (test_foo.Bar) ... ok

----------------------------------------------------------------------
Ran 1 test in 0.000s

OK
"""
    runner = UnittestRunner(None)
    res = runner.load_data(output)
    assert len(res) == 1
    assert res[0].category == Category.OK
    assert res[0].status == 'ok'
    assert res[0].name == 'test_foo.Bar.test1'
    assert res[0].extra_text == []


def test_unittestrunner_load_data_with_exception():
    output = """test1 (test_foo.Bar) ... FAIL
test2 (test_foo.Bar) ... ok

======================================================================
FAIL: test1 (test_foo.Bar)
----------------------------------------------------------------------
Traceback (most recent call last):
  File "/somepath/test_foo.py", line 5, in test1
    self.assertEqual(1, 2)
AssertionError: 1 != 2
"""
    runner = UnittestRunner(None)
    res = runner.load_data(output)
    assert len(res) == 2

    assert res[0].category == Category.FAIL
    assert res[0].status == 'FAIL'
    assert res[0].name == 'test_foo.Bar.test1'
    assert res[0].extra_text[0].startswith('Traceback')
    assert res[0].extra_text[-1].endswith('AssertionError: 1 != 2\n')

    assert res[1].category == Category.OK
    assert res[1].status == 'ok'
    assert res[1].name == 'test_foo.Bar.test2'
    assert res[1].extra_text == []


def test_try_parse_header_with_ok():
    runner = UnittestRunner(None)
    text = 'test_isupper (testfoo.TestStringMethods) ... ok'
    res = runner.try_parse_result(text)
    assert res == ('test_isupper', 'testfoo.TestStringMethods', 'ok', '')


def test_try_parse_header_with_xfail():
    runner = UnittestRunner(None)
    text = 'test_isupper (testfoo.TestStringMethods) ... expected failure'
    res = runner.try_parse_result(text)
    assert res == ('test_isupper', 'testfoo.TestStringMethods',
                   'expected failure', '')


def test_try_parse_header_with_message():
    runner = UnittestRunner(None)
    text = "test_nothing (testfoo.Tests) ... skipped 'msg'"
    res = runner.try_parse_result(text)
    assert res == ('test_nothing', 'testfoo.Tests', 'skipped', 'msg')


def test_try_parse_header_starting_with_digit():
    runner = UnittestRunner(None)
    text = '0est_isupper (testfoo.TestStringMethods) ... ok'
    res = runner.try_parse_result(text)
    assert res is None



# -*- coding: utf-8 -*-
#
# Copyright © 2013 Spyder Project Contributors
# Licensed under the terms of the MIT License
# (see LICENSE.txt for details)
"""Support for Nose framework."""

# Third party imports
from lxml import etree
from spyder.config.base import get_translation

# Local imports
from spyder_unittest.backend.runnerbase import Category, RunnerBase, TestResult

try:
    _ = get_translation("unittest", dirname="spyder_unittest")
except KeyError as error:
    import gettext
    _ = gettext.gettext


class NoseRunner(RunnerBase):
    """Class for running tests within Nose framework."""

    module = 'nose'
    name = 'nose'

    def create_argument_list(self):
        """Create argument list for testing process."""
        return [
            '-m', self.module, '--with-xunit',
            '--xunit-file={}'.format(self.resultfilename)
        ]

    def finished(self):
        """Called when the unit test process has finished."""
        output = self.read_all_process_output()
        testresults = self.load_data()
        self.sig_finished.emit(testresults, output)

    def load_data(self):
        """
        Read and parse unit test results.

        This function reads the unit test results from the file with name
        `self.resultfilename` and parses them. The file should contain the
        test results in JUnitXML format.

        Returns
        -------
        list of TestResult
            Unit test results.
        """
        try:
            data = etree.parse(self.resultfilename).getroot()
        except OSError:
            data = []

        testresults = []
        for testcase in data:
            category = Category.OK
            status = 'ok'
            name = '{}.{}'.format(testcase.get('classname'),
                                  testcase.get('name'))
            message = ''
            time = float(testcase.get('time'))
            extras = []

            for child in testcase:
                if child.tag in ('error', 'failure', 'skipped'):
                    if child.tag == 'skipped':
                        category = Category.SKIP
                    else:
                        category = Category.FAIL
                    status = child.tag
                    type_ = child.get('type')
                    message = child.get('message', default='')
                    if type_ and message:
                        message = '{0}: {1}'.format(type_, message)
                    elif type_:
                        message = type_
                    if child.text:
                        extras.append(child.text)
                elif child.tag in ('system-out', 'system-err'):
                    if child.tag == 'system-out':
                        heading = _('Captured stdout')
                    else:
                        heading = _('Captured stderr')
                    contents = child.text.rstrip('\n')
                    extras.append('----- {} -----\n{}'.format(heading,
                                                              contents))

            extra_text = '\n\n'.join(extras)
            testresults.append(
                TestResult(category, status, name, message, time, extra_text))

        return testresults



# -*- coding: utf-8 -*-
#
# Copyright © 2017 Spyder Project Contributors
# Licensed under the terms of the MIT License
# (see LICENSE.txt for details)
"""Tests for unittestgui.py."""

# Third party imports
from qtpy.QtCore import QModelIndex, QPoint, Qt
from qtpy.QtGui import QContextMenuEvent
import pytest

# Local imports
from spyder_unittest.backend.runnerbase import Category, TestResult
from spyder_unittest.widgets.datatree import (COLORS, TestDataModel,
                                              TestDataView)

try:
    from unittest.mock import Mock
except ImportError:
    from mock import Mock  # Python 2


@pytest.fixture
def view_and_model(qtbot):
    view = TestDataView()
    model = TestDataModel()
    # setModel() before populating testresults because setModel() does a sort
    view.setModel(model)
    res = [TestResult(Category.OK, 'status', 'foo.bar'),
           TestResult(Category.FAIL, 'error', 'foo.bar', 'kadoom', 0,
                      'crash!\nboom!', filename='ham.py', lineno=42)]
    model.testresults = res
    return view, model

def test_contextMenuEvent_calls_exec(view_and_model, monkeypatch):
    # test that a menu is displayed when clicking on an item
    mock_exec = Mock()
    monkeypatch.setattr('spyder_unittest.widgets.datatree.QMenu.exec_', mock_exec)
    view, model = view_and_model
    pos = view.visualRect(model.index(0, 0)).center()
    event = QContextMenuEvent(QContextMenuEvent.Mouse, pos)
    view.contextMenuEvent(event)
    assert mock_exec.called

    # test that no menu is displayed when clicking below the bottom item
    mock_exec.reset_mock()
    pos = view.visualRect(model.index(1, 0)).bottomRight()
    pos += QPoint(0, 1)
    event = QContextMenuEvent(QContextMenuEvent.Mouse, pos)
    view.contextMenuEvent(event)
    assert not mock_exec.called

def test_go_to_test_definition_with_invalid_target(view_and_model, qtbot):
    view, model = view_and_model
    with qtbot.assertNotEmitted(view.sig_edit_goto):
        view.go_to_test_definition(model.index(0, 0))

def test_go_to_test_definition_with_valid_target(view_and_model, qtbot):
    view, model = view_and_model
    with qtbot.waitSignal(view.sig_edit_goto) as blocker:
        view.go_to_test_definition(model.index(1, 0))
    assert blocker.args == ['ham.py', 42]

def test_go_to_test_definition_with_lineno_none(view_and_model, qtbot):
    view, model = view_and_model
    res = model.testresults
    res[1].lineno = None
    model.testresults = res
    with qtbot.waitSignal(view.sig_edit_goto) as blocker:
        view.go_to_test_definition(model.index(1, 0))
    assert blocker.args == ['ham.py', 0]

def test_make_index_canonical_with_index_in_column2(view_and_model):
    view, model = view_and_model
    index = model.index(1, 2)
    res = view.make_index_canonical(index)
    assert res == model.index(1, 0)

def test_make_index_canonical_with_level2_index(view_and_model):
    view, model = view_and_model
    index = model.index(1, 0, model.index(1, 0))
    res = view.make_index_canonical(index)
    assert res == model.index(1, 0)

def test_make_index_canonical_with_invalid_index(view_and_model):
    view, model = view_and_model
    index = QModelIndex()
    res = view.make_index_canonical(index)
    assert res is None

def test_build_context_menu(view_and_model):
    view, model = view_and_model
    menu = view.build_context_menu(model.index(0, 0))
    assert menu.actions()[0].text() == 'Expand'
    assert menu.actions()[1].text() == 'Go to definition'

def test_build_context_menu_with_disabled_entries(view_and_model):
    view, model = view_and_model
    menu = view.build_context_menu(model.index(0, 0))
    assert menu.actions()[0].isEnabled() == False
    assert menu.actions()[1].isEnabled() == False

def test_build_context_menu_with_enabled_entries(view_and_model):
    view, model = view_and_model
    menu = view.build_context_menu(model.index(1, 0))
    assert menu.actions()[0].isEnabled() == True
    assert menu.actions()[1].isEnabled() == True

def test_build_context_menu_with_expanded_entry(view_and_model):
    view, model = view_and_model
    view.expand(model.index(1, 0))
    menu = view.build_context_menu(model.index(1, 0))
    assert menu.actions()[0].text() == 'Collapse'
    assert menu.actions()[0].isEnabled() == True

def test_testdatamodel_using_qtmodeltester(qtmodeltester):
    model = TestDataModel()
    res = [TestResult(Category.OK, 'status', 'foo.bar'),
           TestResult(Category.FAIL, 'error', 'foo.bar', 'kadoom', 0,
                      'crash!\nboom!')]
    model.testresults = res
    qtmodeltester.check(model)

def test_testdatamodel_shows_abbreviated_name_in_table(qtbot):
    model = TestDataModel()
    res = TestResult(Category.OK, 'status', 'foo.bar', '', 0, '')
    model.testresults = [res]
    index = model.index(0, 1)
    assert model.data(index, Qt.DisplayRole) == 'f.bar'

def test_testdatamodel_shows_full_name_in_tooltip(qtbot):
    model = TestDataModel()
    res = TestResult(Category.OK, 'status', 'foo.bar', '', 0, '')
    model.testresults = [res]
    index = model.index(0, 1)
    assert model.data(index, Qt.ToolTipRole) == 'foo.bar'

def test_testdatamodel_shows_time(qtmodeltester):
    model = TestDataModel()
    res = TestResult(Category.OK, 'status', 'foo.bar', time=0.0012345)
    model.testresults = [res]
    index = model.index(0, 3)
    assert model.data(index, Qt.DisplayRole) == '1.23'
    assert model.data(index, Qt.TextAlignmentRole) == Qt.AlignRight

def test_testdatamodel_shows_time_when_zero(qtmodeltester):
    model = TestDataModel()
    res = TestResult(Category.OK, 'status', 'foo.bar', time=0)
    model.testresults = [res]
    assert model.data(model.index(0, 3), Qt.DisplayRole) == '0.00'

def test_testdatamodel_shows_time_when_blank(qtmodeltester):
    model = TestDataModel()
    res = TestResult(Category.OK, 'status', 'foo.bar')
    model.testresults = [res]
    assert model.data(model.index(0, 3), Qt.DisplayRole) == ''

def test_testdatamodel_data_background():
    model = TestDataModel()
    res = [TestResult(Category.OK, 'status', 'foo.bar'),
           TestResult(Category.FAIL, 'error', 'foo.bar', 'kadoom')]
    model.testresults = res
    index = model.index(0, 0)
    assert model.data(index, Qt.BackgroundRole) == COLORS[Category.OK]
    index = model.index(1, 2)
    assert model.data(index, Qt.BackgroundRole) == COLORS[Category.FAIL]

def test_testdatamodel_data_userrole():
    model = TestDataModel()
    res = [TestResult(Category.OK, 'status', 'foo.bar', filename='somefile',
                      lineno=42)]
    model.testresults = res
    index = model.index(0, 0)
    assert model.data(index, Qt.UserRole) == ('somefile', 42)

def test_testdatamodel_add_tests(qtbot):
    def check_args1(parent, begin, end):
        return not parent.isValid() and begin == 0 and end == 0

    def check_args2(parent, begin, end):
        return not parent.isValid() and begin == 1 and end == 1

    model = TestDataModel()
    assert model.testresults == []

    result1 = TestResult(Category.OK, 'status', 'foo.bar')
    with qtbot.waitSignals([model.rowsInserted, model.sig_summary],
                           check_params_cbs=[check_args1, None],
                           raising=True):
        model.add_testresults([result1])
    assert model.testresults == [result1]

    result2 = TestResult(Category.FAIL, 'error', 'foo.bar', 'kadoom')
    with qtbot.waitSignals([model.rowsInserted, model.sig_summary],
                           check_params_cbs=[check_args2, None],
                           raising=True):
        model.add_testresults([result2])
    assert model.testresults == [result1, result2]


def test_testdatamodel_replace_tests(qtbot):
    def check_args(topLeft, bottomRight, *args):
        return (topLeft.row() == 0
                and topLeft.column() == 0
                and not topLeft.parent().isValid()
                and bottomRight.row() == 0
                and bottomRight.column() == 3
                and not bottomRight.parent().isValid())

    model = TestDataModel()
    result1 = TestResult(Category.OK, 'status', 'foo.bar')
    model.testresults = [result1]
    result2 = TestResult(Category.FAIL, 'error', 'foo.bar', 'kadoom')
    with qtbot.waitSignals([model.dataChanged, model.sig_summary],
                           check_params_cbs=[check_args, None],
                           raising=True):
        model.update_testresults([result2])
    assert model.testresults == [result2]

STANDARD_TESTRESULTS = [
    TestResult(Category.OK, 'status', 'foo.bar', time=2),
    TestResult(Category.FAIL, 'failure', 'fu.baz', 'kaboom',time=1),
    TestResult(Category.FAIL, 'error', 'fu.bar', 'boom')]

def test_testdatamodel_sort_by_status_ascending(qtbot):
    model = TestDataModel()
    model.testresults = STANDARD_TESTRESULTS[:]
    with qtbot.waitSignal(model.modelReset):
        model.sort(0, Qt.AscendingOrder)
    expected = [STANDARD_TESTRESULTS[k] for k in [2, 1, 0]]
    assert model.testresults == expected

def test_testdatamodel_sort_by_status_descending():
    model = TestDataModel()
    model.testresults = STANDARD_TESTRESULTS[:]
    model.sort(0, Qt.DescendingOrder)
    expected = [STANDARD_TESTRESULTS[k] for k in [0, 1, 2]]
    assert model.testresults == expected

def test_testdatamodel_sort_by_name():
    model = TestDataModel()
    model.testresults = STANDARD_TESTRESULTS[:]
    model.sort(1, Qt.AscendingOrder)
    expected = [STANDARD_TESTRESULTS[k] for k in [0, 2, 1]]
    assert model.testresults == expected

def test_testdatamodel_sort_by_message():
    model = TestDataModel()
    model.testresults = STANDARD_TESTRESULTS[:]
    model.sort(2, Qt.AscendingOrder)
    expected = [STANDARD_TESTRESULTS[k] for k in [0, 2, 1]]
    assert model.testresults == expected

def test_testdatamodel_sort_by_time():
    model = TestDataModel()
    model.testresults = STANDARD_TESTRESULTS[:]
    model.sort(3, Qt.AscendingOrder)
    expected = [STANDARD_TESTRESULTS[k] for k in [2, 1, 0]]
    assert model.testresults == expected



# -*- coding: utf-8 -*-
#
# Copyright © 2017 Spyder Project Contributors
# Licensed under the terms of the MIT License
# (see LICENSE.txt for details)
"""Support for unittest framework."""

# Standard library imports
import re

# Local imports
from spyder_unittest.backend.runnerbase import Category, RunnerBase, TestResult


class UnittestRunner(RunnerBase):
    """Class for running tests with unittest module in standard library."""

    module = 'unittest'
    name = 'unittest'

    def create_argument_list(self):
        """Create argument list for testing process."""
        return ['-m', self.module, 'discover', '-v']

    def finished(self):
        """
        Called when the unit test process has finished.

        This function reads the results and emits `sig_finished`.
        """
        output = self.read_all_process_output()
        testresults = self.load_data(output)
        self.sig_finished.emit(testresults, output)

    def load_data(self, output):
        """
        Read and parse output from unittest module.

        Returns
        -------
        list of TestResult
            Unit test results.
        """
        res = []
        lines = output.splitlines()
        line_index = 0
        test_index = None

        while line_index < len(lines):
            data = self.try_parse_result(lines[line_index])
            if data:
                if data[2] == 'ok':
                    cat = Category.OK
                elif data[2] == 'FAIL' or data[2] == 'ERROR':
                    cat = Category.FAIL
                else:
                    cat = Category.SKIP
                name = '{}.{}'.format(data[1], data[0])
                tr = TestResult(category=cat, status=data[2], name=name,
                                message=data[3])
                res.append(tr)
                line_index += 1
                test_index = -1
                continue

            data = self.try_parse_exception_header(lines, line_index)
            if data:
                line_index = data[0]
                test_index = next(
                    i for i, tr in enumerate(res)
                    if tr.name == '{}.{}'.format(data[2], data[1]))

            data = self.try_parse_footer(lines, line_index)
            if data:
                line_index = data
                test_index = -1
                continue

            if test_index is not None:
                res[test_index].extra_text.append(lines[line_index] + '\n')
                line_index += 1

        return res

    def try_parse_result(self, line):
        """
        Try to parse a line of text as a test result.

        Returns
        -------
        tuple of str or None
            If line represents a test result, then return a tuple with four
            strings: the name of the test function, the name of the test class,
            the test result, and the reason (if no reason is given, the fourth
            string is empty). Otherwise, return None.
        """
        regexp = (r'([^\d\W]\w*) \(([^\d\W][\w.]*)\) \.\.\. '
                  '(ok|FAIL|ERROR|skipped|expected failure|unexpected success)'
                  "( '([^']*)')?\Z")
        match = re.match(regexp, line)
        if match:
            msg = match.groups()[4] or ''
            return match.groups()[:3] + (msg, )
        else:
            return None

    def try_parse_exception_header(self, lines, line_index):
        """
        Try to parse the header of an exception in unittest output.

        Returns
        -------
        (int, str, str) or None
            If an exception header is parsed successfully, then return a tuple
            with the new line index, the name of the test function, and the
            name of the test class. Otherwise, return None.
        """
        if lines[line_index] != '':
            return None
        if not all(char == '=' for char in lines[line_index + 1]):
            return None
        regexp = r'\w+: ([^\d\W]\w*) \(([^\d\W][\w.]*)\)\Z'
        match = re.match(regexp, lines[line_index + 2])
        if not match:
            return None
        if not all(char == '-' for char in lines[line_index + 3]):
            return None
        return (line_index + 4, ) + match.groups()

    def try_parse_footer(self, lines, line_index):
        """
        Try to parse footer of unittest output.

        Returns
        -------
        int or None
            New line index if footer is parsed successfully, None otherwise
        """
        if lines[line_index] != '':
            return None
        if not all(char == '-' for char in lines[line_index + 1]):
            return None
        if not re.match(r'^Ran [\d]+ tests? in', lines[line_index + 2]):
            return None
        if lines[line_index + 3] != '':
            return None
        return line_index + 5



# -*- coding: utf-8 -*-
#
# Copyright © 2017 Spyder Project Contributors
# Licensed under the terms of the MIT License
# (see LICENSE.txt for details)
"""Tests for abbreviator.py"""

# Local imports
from spyder_unittest.backend.abbreviator import Abbreviator


def test_abbreviator_with_one_word():
    abb = Abbreviator()
    abb.add('ham')
    assert abb.abbreviate('ham') == 'ham'

def test_abbreviator_with_one_word_with_two_components():
    abb = Abbreviator()
    abb.add('ham.spam')
    assert abb.abbreviate('ham.spam') == 'h.spam'

def test_abbreviator_with_one_word_with_three_components():
    abb = Abbreviator()
    abb.add('ham.spam.eggs')
    assert abb.abbreviate('ham.spam.eggs') == 'h.s.eggs'

def test_abbreviator_without_common_prefix():
    abb = Abbreviator(['ham.foo', 'spam.foo'])
    assert abb.abbreviate('ham.foo') == 'h.foo'
    assert abb.abbreviate('spam.foo') == 's.foo'

def test_abbreviator_with_prefix():
    abb = Abbreviator(['test_ham.x', 'test_spam.x'])
    assert abb.abbreviate('test_ham.x') == 'test_h.x'
    assert abb.abbreviate('test_spam.x') == 'test_s.x'

def test_abbreviator_with_first_word_prefix_of_second():
    abb = Abbreviator(['ham.x', 'hameggs.x'])
    assert abb.abbreviate('ham.x') == 'ham.x'
    assert abb.abbreviate('hameggs.x') == 'hame.x'

def test_abbreviator_with_second_word_prefix_of_first():
    abb = Abbreviator(['hameggs.x', 'ham.x'])
    assert abb.abbreviate('hameggs.x') == 'hame.x'
    assert abb.abbreviate('ham.x') == 'ham.x'

def test_abbreviator_with_three_words():
    abb = Abbreviator(['hamegg.x', 'hameggs.x', 'hall.x'])
    assert abb.abbreviate('hamegg.x') == 'hamegg.x'
    assert abb.abbreviate('hameggs.x') == 'hameggs.x'
    assert abb.abbreviate('hall.x') == 'hal.x'

def test_abbreviator_with_multilevel():
    abb = Abbreviator(['ham.eggs.foo', 'ham.spam.bar', 'eggs.ham.foo',
                       'eggs.hamspam.bar'])
    assert abb.abbreviate('ham.eggs.foo') == 'h.e.foo'
    assert abb.abbreviate('ham.spam.bar') == 'h.s.bar'
    assert abb.abbreviate('eggs.ham.foo') == 'e.ham.foo'
    assert abb.abbreviate('eggs.hamspam.bar') == 'e.hams.bar'



# -*- coding: utf-8 -*-
#
# Copyright © 2013 Spyder Project Contributors
# Licensed under the terms of the MIT License
# (see LICENSE.txt for details)
r"""
Reader and writer for sending stream of python objects using JSON.

These classes can be used to send Python objects (specifically, ints, floats,
strings, bools, lists, dictionaries or None) over a text stream. Partially
received objects are correctly handled.

Since multiple JSON-encoded objects cannot simply concatenated (i.e., JSON is
not a framed protocol), every object is sent over the text channel in the
format "N \n s \n", where the string s is its JSON encoding and N is the length
of s.
"""

# Standard library imports
import json

# Third party imports
from spyder.py3compat import PY2, to_text_string


class JSONStreamWriter:
    """
    Writer for sending stream of python objects using JSON.

    This class can be used to send a stream of python objects over a text
    stream using JSON. It is the responsibility of the caller to open and
    close the stream.

    Attributes
    ----------
    stream : TextIOBase
        text stream that the objects are sent over.
    """

    def __init__(self, stream):
        """Constructor."""
        self.stream = stream

    def write(self, obj):
        """
        Write Python object to the stream and flush.

        Arguments
        ---------
        obj : object
            Object to be written. The type should be supported by JSON (i.e.,
            int, float, str, bool, list, dict or None).
        """
        txt = json.dumps(obj)
        if PY2:
            txt = to_text_string(txt)
        self.stream.write(to_text_string(len(txt)) + '\n')
        self.stream.write(txt + '\n')
        self.stream.flush()


class JSONStreamReader:
    """
    Reader for sending stream of Python objects using JSON.

    This class is used to receive a stream sent by JSONStreamWriter.

    Attributes
    ----------
    buffer : str
       Text encoding an object that has not been completely received yet.
    """

    def __init__(self):
        """Constructor."""
        self.buffer = ''

    def consume(self, txt):
        """
        Decode given text and return list of objects encoded in it.

        If only a part of the encoded text of an object is passed, then it is
        stored and combined with the remainder in the next call.
        """
        index = 0
        res = []
        txt = self.buffer + txt
        while index < len(txt):
            has_r = False  # whether line ends with \r\n or \n
            end_of_line1 = txt.find('\n', index)
            try:
                len_encoding = int(txt[index:end_of_line1])
            except ValueError:
                raise ValueError('txt = %s  index = %d  end_of_line1 = %d'
                                 % (repr(txt), index, end_of_line1))
            if end_of_line1 + len_encoding + 2 > len(txt):  # 2 for two \n
                break
            if txt[end_of_line1 + len_encoding + 1] == '\r':
                if end_of_line1 + len_encoding + 3 > len(txt):
                    break
                else:
                    has_r = True
            encoding = txt[end_of_line1 + 1:end_of_line1 + len_encoding + 1]
            res.append(json.loads(encoding))
            index = end_of_line1 + len_encoding + 2
            if has_r:
                index += 1
        self.buffer = txt[index:]
        return res



# -*- coding: utf-8 -*-
#
# Copyright © 2017 Spyder Project Contributors
# Licensed under the terms of the MIT License
# (see LICENSE.txt for details)
"""Tests for pytestworker.py"""

# Standard library imports
import os

# Third party imports
import pytest

# Local imports
from spyder_unittest.backend.jsonstream import JSONStreamWriter
from spyder_unittest.backend.pytestworker import SpyderPlugin, main

try:
    from unittest.mock import call, create_autospec, Mock
except ImportError:
    from mock import call, create_autospec, Mock  # Python 2


class EmptyClass:
    pass


@pytest.fixture
def plugin():
    mock_writer = create_autospec(JSONStreamWriter)
    return SpyderPlugin(mock_writer)

def test_spyderplugin_test_collectreport_with_success(plugin):
    report = EmptyClass()
    report.outcome = 'success'
    report.nodeid = 'foo.py::bar'
    plugin.pytest_collectreport(report)
    plugin.writer.write.assert_not_called()

def test_spyderplugin_test_collectreport_with_failure(plugin):
    report = EmptyClass()
    report.outcome = 'failed'
    report.nodeid = 'foo.py::bar'
    report.longrepr = EmptyClass()
    report.longrepr.longrepr = 'message'
    plugin.pytest_collectreport(report)
    plugin.writer.write.assert_called_once_with({
        'event': 'collecterror',
        'nodeid': 'foo.py::bar',
        'longrepr': 'message'
    })

def test_spyderplugin_test_itemcollected(plugin):
    testitem = EmptyClass()
    testitem.name = 'bar'
    testitem.parent = EmptyClass()
    testitem.parent.name = 'foo.py'
    testitem.parent.parent = EmptyClass
    testitem.parent.parent.name = 'notused'
    testitem.parent.parent.parent = None
    plugin.pytest_itemcollected(testitem)
    plugin.writer.write.assert_called_once_with({
        'event': 'collected',
        'nodeid': 'foo.py::bar'
    })

def standard_logreport():
    report = EmptyClass()
    report.when = 'call'
    report.outcome = 'passed'
    report.nodeid = 'foo.py::bar'
    report.duration = 42
    report.sections = []
    report.longrepr = ''
    report.location = ('foo.py', 24, 'bar')
    return report

def test_spyderplugin_runtest_logreport(plugin):
    report = standard_logreport()
    plugin.pytest_runtest_logreport(report)
    plugin.writer.write.assert_called_once_with({
        'event': 'logreport',
        'when': 'call',
        'outcome': 'passed',
        'nodeid': 'foo.py::bar',
        'duration': 42,
        'sections': [],
        'filename': 'foo.py',
        'lineno': 24
    })

def test_spyderplugin_runtest_logreport_passes_longrepr(plugin):
    report = standard_logreport()
    report.longrepr = 15
    plugin.pytest_runtest_logreport(report)
    plugin.writer.write.assert_called_once_with({
        'event': 'logreport',
        'when': 'call',
        'outcome': 'passed',
        'nodeid': 'foo.py::bar',
        'duration': 42,
        'sections': [],
        'filename': 'foo.py',
        'lineno': 24,
        'longrepr': '15'
    })

def test_spyderplugin_runtest_logreport_with_longrepr_tuple(plugin):
    report = standard_logreport()
    report.longrepr = ('ham', 'spam')
    plugin.pytest_runtest_logreport(report)
    plugin.writer.write.assert_called_once_with({
        'event': 'logreport',
        'when': 'call',
        'outcome': 'passed',
        'nodeid': 'foo.py::bar',
        'duration': 42,
        'sections': [],
        'filename': 'foo.py',
        'lineno': 24,
        'longrepr': ('ham', 'spam')
    })

def test_spyderplugin_runtest_logreport_passes_wasxfail(plugin):
    report = standard_logreport()
    report.wasxfail = ''
    plugin.pytest_runtest_logreport(report)
    plugin.writer.write.assert_called_once_with({
        'event': 'logreport',
        'when': 'call',
        'outcome': 'passed',
        'nodeid': 'foo.py::bar',
        'duration': 42,
        'sections': [],
        'filename': 'foo.py',
        'lineno': 24,
        'wasxfail': ''
    })

def test_spyderplugin_runtest_logreport_passes_message(plugin):
    class MockLongrepr:
        def __init__(self):
            self.reprcrash = EmptyClass()
            self.reprcrash.message = 'msg'
        def __str__(self):
            return 'text'

    report = standard_logreport()
    report.longrepr = MockLongrepr()
    plugin.pytest_runtest_logreport(report)
    plugin.writer.write.assert_called_once_with({
        'event': 'logreport',
        'when': 'call',
        'outcome': 'passed',
        'nodeid': 'foo.py::bar',
        'duration': 42,
        'sections': [],
        'filename': 'foo.py',
        'lineno': 24,
        'longrepr': 'text',
        'message': 'msg'
    })

def test_spyderplugin_runtest_logreport_ignores_teardown_passed(plugin):
    report = standard_logreport()
    report.when = 'teardown'
    plugin.pytest_runtest_logreport(report)
    plugin.writer.write.assert_not_called()

def test_main_captures_stdout_and_stderr(monkeypatch):
    def mock_main(args, plugins):
        print('output')
    monkeypatch.setattr(
        'spyder_unittest.backend.pytestworker.pytest.main', mock_main)

    mock_writer = create_autospec(JSONStreamWriter)
    MockJSONStreamWriter = Mock(return_value=mock_writer)
    monkeypatch.setattr(
        'spyder_unittest.backend.pytestworker.JSONStreamWriter',
        MockJSONStreamWriter)

    main(None)
    mock_writer.write.assert_called_once_with({
            'event': 'finished', 'stdout': 'output\n'})

def test_pytestworker_integration(monkeypatch, tmpdir):
    os.chdir(tmpdir.strpath)
    testfilename = tmpdir.join('test_foo.py').strpath
    with open(testfilename, 'w') as f:
        f.write("def test_ok(): assert 1+1 == 2\n"
                "def test_fail(): assert 1+1 == 3\n")

    mock_writer = create_autospec(JSONStreamWriter)
    MockJSONStreamWriter = Mock(return_value=mock_writer)
    monkeypatch.setattr(
        'spyder_unittest.backend.pytestworker.JSONStreamWriter',
        MockJSONStreamWriter)
    main([testfilename])

    args = mock_writer.write.call_args_list

    assert args[0][0][0]['event'] == 'collected'
    assert args[0][0][0]['nodeid'] == 'test_foo.py::test_ok'

    assert args[1][0][0]['event'] == 'collected'
    assert args[1][0][0]['nodeid'] == 'test_foo.py::test_fail'

    assert args[2][0][0]['event'] == 'starttest'
    assert args[2][0][0]['nodeid'] == 'test_foo.py::test_ok'

    assert args[3][0][0]['event'] == 'logreport'
    assert args[3][0][0]['when'] == 'call'
    assert args[3][0][0]['outcome'] == 'passed'
    assert args[3][0][0]['nodeid'] == 'test_foo.py::test_ok'
    assert args[3][0][0]['sections'] == []
    assert args[3][0][0]['filename'] == 'test_foo.py'
    assert args[3][0][0]['lineno'] == 0
    assert 'duration' in args[3][0][0]

    assert args[4][0][0]['event'] == 'starttest'
    assert args[4][0][0]['nodeid'] == 'test_foo.py::test_fail'

    assert args[5][0][0]['event'] == 'logreport'
    assert args[5][0][0]['when'] == 'call'
    assert args[5][0][0]['outcome'] == 'failed'
    assert args[5][0][0]['nodeid'] == 'test_foo.py::test_fail'
    assert args[5][0][0]['sections'] == []
    assert args[5][0][0]['filename'] == 'test_foo.py'
    assert args[5][0][0]['lineno'] == 1
    assert 'duration' in args[5][0][0]

    assert args[6][0][0]['event'] == 'finished'
    assert 'pytest' in args[6][0][0]['stdout']
